@@ -516,7 +516,6 @@ def rotate2X(v):
     return Rotation
 
 def LengthOrder(v):
-    
     nonZeroPos = np.logical_not(np.isclose(v,0.0))
     if np.sum(nonZeroPos)==1:
         Rv = v/np.linalg.norm(v)
@@ -1153,4 +1152,21 @@ def DSpacing(TwoTheta,Energy=None,Wavelength=None,K=None,degrees = True):
 def rescale(x,f=0.5):
     return f*x*x*np.sign(x)
 def scale(x,f=0.5):
-    return np.sqrt(np.abs(x)/f)*np.sign(x)
\ No newline at end of file
+    return np.sqrt(np.abs(x)/f)*np.sign(x)
+
+def getNext(l,delete=True):
+    """get elements in itterator and delete list depending on flag
+    
+    Args:
+        - l (itterable): List or liste-like to loop over
+
+    Kwargs: delete (bool): If true, itteratively delete elements in list, else keep elements (default True)
+    
+    """
+    if delete:
+        while len(l)>0:
+            yield l[0]
+            del l[0]
+    else:
+        for entry in l:
+            yield entry
\ No newline at end of file
